/******************************************************************************/
/*                                                                            */
/*                            Checkers Definitions                            */
/*                            -------------------                             */
/*                                                                            */
/* Written by: John Thornley, Computer Science Dept., Caltech.                */
/* Last modified: Monday 24th January 1994.                                   */
/*                                                                            */
/* Purpose: Definitions of functions for the game Checkers.                   */
/*                                                                            */
/******************************************************************************/


#include "boolean.h"
#include "checkers.h"
#include "archgtus.h"
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>

/* should I make 0 adjacent to 1-it causes a problem with checking jumps */
const square adjacent_squares[NUM_SQUARES][NUM_DIRECTIONS] = {
  {0,0,0,0},
  {0,0,5,6},      {0,0,6,7},      {0,0,7,8},      {0,0,8,0},
  {0,1,0,9},      {1,2,9,10},     {2,3,10,11},    {3,4,11,12},
  {5,6,13,14},    {6,7,14,15},    {7,8,15,16},    {8,0,16,0},
  {0,9,0,17},     {9,10,17,18},   {10,11,18,19},  {11,12,19,20},
  {13,14,21,22},  {14,15,22,23},  {15,16,23,24},  {16,0,24,0},
  {0,17,0,25},    {17,18,25,26},  {18,19,26,27},  {19,20,27,28},
  {21,22,29,30},  {22,23,30,31},  {23,24,31,32},  {24,0,32,0},
  {0,25,0,0},     {25,26,0,0},    {26,27,0,0},    {27,28,0,0},
};

boolean last_rank[NUM_SQUARES]={
  false,
  true,  true,  true,  true,
  false, false, false, false,
  false, false, false, false,
  false, false, false, false,
  false, false, false, false,
  false, false, false, false,
  false, false, false, false,
  true,  true,  true,  true,
};

/* depends on order of enumerated type-black_man, black_king, */
/* white_man, white_king.                                     */

int position_weight[NUM_PIECE_TYPES][NUM_SQUARES] = {
  {0,0,0,0,0,0,1,1,1,2,2,2,1,2,3,3,3,4,4,4,3,4,5,5,5,6,6,6,5,0,0,0,0},
  {0,0,0,0,0,0,2,2,2,2,4,4,0,0,4,4,2,2,4,4,0,0,4,4,2,2,2,2,0,0,0,0,0},
  {0,0,0,0,0,-5,-6,-6,-6,-5,-5,-5,-4,-3,-4,-4,-4,-3,-3,-3,
     -2,-1,-2,-2,-2,-1,-1,-1,0,0,0,0,0},
  {0,0,0,0,0,0,-2,-2,-2,-2,-4,-4,0,0,-4,-4,-2,-2,-4,-4,0,0,-4,
     -4,-2,-2,-2,-2,0,0,0,0,0},
};

/*--------------------------------------------------------------------------*/
/*                      MISCELLANEOUS PRIVATE FUNCTIONS                     */
/*--------------------------------------------------------------------------*/
boolean in_last_rank_of(square the_square, color the_color) {
  if(the_color == black) {
    if(the_square==1 || the_square ==2 || the_square==3 ||the_square==4) {
      return true;
    }
  } else {
    if(the_square==29 || the_square==30 || the_square==31 || the_square==32) {
      return true;
    }
  }
  return false;
}

/*--------------------------------------------------------------------------*/
occupant man_of_color(color c) {
    occupant result=black_man;
    switch (c) {
      case black:
	result = black_man;
	break;
      case white:
	result = white_man;
	break;
    }
    return result;
}

/*--------------------------------------------------------------------------*/
occupant king_of_color(color c) {
    occupant result=black_king;
    switch (c) {
      case black:
	result = black_king;
	break;
      case white:
	result = white_king;
	break;
    }
    return result;
}


/*--------------------------------------------------------------------------*/
int sign(int i) {
    if (i < 0)
	return -1;
    else if (i > 0)
	return 1;
    else
	return 0;
}

/*--------------------------------------------------------------------------*/
color color_of(occupant the_occupant) {
  switch(the_occupant) {
     case black_man:
     case black_king:
	 return black;
     case white_man:
     case white_king:
	 return white;
     default:
	assert(false);
	/* only to compile w/o warning */
	return white;
  }
}

/*--------------------------------------------------------------------------*/
color opposite_color(color the_color) {
  switch(the_color) {
    case black:
      return white;
    case white:
      return black;
    default:
      assert(false);
      /* only to compile w/o warning */
      return white;
  }
}

/*--------------------------------------------------------------------------*/
/*                        VALIDITY CHECKING FUNCTIONS                       */
/*--------------------------------------------------------------------------*/

boolean valid_color(color the_color) {
    return the_color == black || the_color == white;
}

/*--------------------------------------------------------------------------*/

/* SHOULD I count off_board ??????? */
boolean valid_occupant(occupant the_occupant) {
    return the_occupant == empty ||
	   the_occupant == black_man || the_occupant == white_man ||
	   the_occupant == black_king || the_occupant == white_king;
}

/*--------------------------------------------------------------------------*/
/* checks that board[0] is off_board & that all other pieces are valid-empty*/
/* or white or black.  It also checks that there is no more than 12 black   */
/* 12 white pieces.  Could add more rigorous checks later.                  */

boolean valid_board(occupant board[33]) {
  boolean result = true;
  int i, black_count=0, white_count=0;
  if(board[0] != off_board)
    return(false);
  for (i=1; i<33; i++) {
      if (board[i] == white_man || board[i] == white_king) {
	white_count++;
      }
      if(board[i] == black_man || board[i] == black_king) {
	black_count++;
      }
      result = result && valid_occupant(board[i]);
  }
  if (white_count > 12 || black_count > 12)
     return(false);
  return(result);
}

/*--------------------------------------------------------------------------*/

boolean valid_game(game the_game) {
    return valid_board(the_game.board) && valid_color(the_game.to_move);
}

/*-------------------------------------------------------------------------*/
/* check_jump_adjacency                                                    */
/* Checks that the jump is in the right direction for the given piece and  */
/* that the jump is along an adjacent diagonal.   It doesn't allow         */
/*  a move to a Non_Existent Square  */
/* (off the board).  It also ensures that the move is in the allowed       */
/* direction for the piece.  If the piece to move is not a man or king     */
/* it exits with an assert(false)-better way to exit???                    */
/*-------------------------------------------------------------------------*/

boolean check_jump_adjacency(occupant board[NUM_SQUARES], square move_from,
			square move_to) {
  if(move_to == NON_EXISTENT_SQUARE)
    return false;
  switch(board[move_from]) {
    case black_man:
      if(adjacent_squares[adjacent_squares[move_from][down_left]][down_left]
	   == move_to ||
	   adjacent_squares[adjacent_squares[move_from][down_right]][down_right]
	   == move_to)
	return true;
      break;
    case white_man:
      if(adjacent_squares[adjacent_squares[move_from][up_left]][up_left]
	  == move_to ||
	  adjacent_squares[adjacent_squares[move_from][up_right]][up_right]
	  == move_to)
	return true;
      break;
    case black_king:
    case white_king:
      if(adjacent_squares[adjacent_squares[move_from][down_left]][down_left]
	  == move_to ||
	  adjacent_squares[adjacent_squares[move_from][down_right]][down_right]
	  ==move_to ||
	  adjacent_squares[adjacent_squares[move_from][up_left]][up_left]
	  == move_to ||
	  adjacent_squares[adjacent_squares[move_from][up_right]][up_right]
	  == move_to)
	return true;
      break;
    default:
      assert(false);
  }
  return false;
}

/*-------------------------------------------------------------------------*/
/* check_adjacency:                                                        */
/* Predicate that checks that the square to move_to is adjacent to the     */
/* square to move_from.  It doesn't allow a move to a Non_Existent Square  */
/* (off the board).  It also ensures that the move is in the allowed       */
/* direction for the piece.  If the piece to move is not a man or king     */
/* it exits with an assert(false)-better way to exit???                    */
/*-------------------------------------------------------------------------*/
boolean check_adjacency(occupant board[NUM_SQUARES], square  move_from,
			square  move_to) {
  if(move_to == NON_EXISTENT_SQUARE)
    return false;
  switch(board[move_from]) {
    case black_man:
      if(adjacent_squares[move_from][down_left] == move_to ||
	 adjacent_squares[move_from][down_right] == move_to)
	return true;
      break;
    case white_man:
      if(adjacent_squares[move_from][up_left] == move_to ||
	 adjacent_squares[move_from][up_right] == move_to)
	return true;
      break;
    case black_king:
    case white_king:
      if(adjacent_squares[move_from][down_left] == move_to ||
	 adjacent_squares[move_from][down_right] ==move_to ||
	 adjacent_squares[move_from][up_left] == move_to ||
	 adjacent_squares[move_from][up_right] == move_to)
	return true;
      break;
    default:
      assert(false);
  }
  return false;
}

/*-------------------------------------------------------------------*/
/* valid_move_test                                                   */
/* If the move isn't a jump, it checks that the move is to an empty  */
/* square and that the move is along a diagonal in an appropriate    */
/* direction for the piece.  If the move is a jump, it checks for    */
/* all jumps that the place to jump_to is empty, that it jumps over  */
/* the opposite color, and that the jump is along an appropriate     */
/* diagonal.  It doesn't check if the piece has a compulsory jump.   */
/* don't require the hop_tos to match b/c don't fill it up in        */
/* scan move.  YES!! THAT'S LIKELY THE PROBLEM!!                     */
/*-------------------------------------------------------------------*/
boolean valid_move_test(occupant board[33], color to_move, move the_move) {
  int i;
  if(!the_move.is_jump) {
      /* Verify move to an adjacent, empty square */
      if(board[the_move.move_to]==empty && check_adjacency(board,
			     the_move.move_from, the_move.move_to))
	  return true;
  } else {
      /* Verify jumping to adjacent, empty square of opposite color */
      for(i=0; i<the_move.num_hops; i++) {
	 if(board[the_move.hops[i].hop_to]==empty &&
	     color_of(board[the_move.hops[i].hop_over])!=to_move &&
	     check_jump_adjacency(board, the_move.hops[i].hop_from,
				  the_move.hops[i].hop_to))
	    return true;
      }
  }
  return false;
}

/* ----------------------------------------------------------------------*/
boolean is_equal_move(move left, move right) {
  int i;
  if(left.is_jump) {
    /* check jumps */
    if(!right.is_jump || left.num_hops != right.num_hops)
      return false;
    for(i=0; i<left.num_hops; i++) {
      if(!((left.hops[i].hop_from == right.hops[i].hop_from) &&
	    (left.hops[i].hop_to == right.hops[i].hop_to)))
	return false;
    }
    return true;
  } else
    /* check moves */
    return(!right.is_jump && left.move_from == right.move_from &&
			     left.move_to == right.move_to);
}


/* ----------------------------------------------------------------------*/
boolean move_in_generated_list(move chosen_move, moves move_list) {
  int i;
  /* case where user moves before minimax gets called-assumed that  */
  /* after the first move, the game will be stopped before reaching */
  /* here. */
  if(move_list.length == 0)
    return true;
  for(i=0; i<move_list.length; i++) {
    if(is_equal_move(move_list.list[i], chosen_move)) {
      return true;
    }
  }
  return false;
}


/*----------------------------------------------------------------------------*/
boolean valid_score(score the_score) {
    boolean result;
    int piece_diff, max_mobility;
    piece_diff = (the_score.black_pieces - the_score.white_pieces);
    max_mobility = ( (MAX_MOVES * MOBILITY_VALUE) + (BRIDGE_VALUE) +
		    (MAX_PIECES *6));
    if (the_score.status != black_win && the_score.status != white_win &&
	the_score.status != unfinished  )
	result = false;
    else if (piece_diff < -(MAX_PIECES*KING_VALUE +1) ||
	     (MAX_PIECES*KING_VALUE + 1) < piece_diff)
	result = false;
    else if (the_score.mobility < -(max_mobility) || max_mobility < the_score.mobility)
	result = false;
    else
	result = true;
    return result;
}

/*----------------------------------------------------------------------------*/
/*                          GAME AND MOVE FUNCTIONS                           */
/*----------------------------------------------------------------------------*/

void start_game(game *the_game) {
    int i;
    the_game->to_move = black;
    the_game->board[0] =off_board;
    for(i=1; i<13; i++) {
      the_game->board[i] = black_man;
    }
    for(; i<21; i++) {
      the_game->board[i] = empty;
    }
    for(; i<33; i++) {
      the_game->board[i] = white_man;
    }
}


/*------------------------------------------------------------------------*/
void try_move(square square_from, game *the_game,  moves *move_list,
	      square square_to) {
  if(the_game->board[square_to] == empty) {
      move_list->list[move_list->length].is_jump = false;
      move_list->list[move_list->length].move_from = square_from;
      move_list->list[move_list->length].move_to = square_to;
      move_list->length++;
  }
}

/*------------------------------------------------------------------------*/
void generate_non_jumps(game *the_game, moves *move_list) {
  piece my_king;
  direction forward_right, forward_left, backward_left, backward_right;
  square the_square;
  if(the_game->to_move == black) {
    my_king = black_king;
    forward_left = down_right;
    forward_right = down_left;
    backward_left = up_right;
    backward_right = up_left;
  } else {
    my_king = white_king;
    forward_left = up_left;
    forward_right = up_right;
    backward_left = down_left;
    backward_right = down_right;
  }
  for(the_square = 1; the_square < NUM_SQUARES; the_square++) {
    /* for testing */
    if(the_game->board[the_square] != empty) {
      if(color_of(the_game->board[the_square]) == the_game->to_move) {
	try_move(the_square,the_game,move_list,
		 adjacent_squares[the_square][forward_left]);
	try_move(the_square,the_game,move_list,
		 adjacent_squares[the_square][forward_right]);
	if(the_game->board[the_square] == my_king) {
	  try_move(the_square,the_game,move_list,
		   adjacent_squares[the_square][backward_left]);
	  try_move(the_square,the_game,move_list,
		   adjacent_squares[the_square][backward_right]);
	}
      }
    }
  }
}

/*--------------------------------------------------------------------------*/
boolean try_jump(occupant board[NUM_SQUARES], square jump_from,
		 direction the_direction, move *hops_so_far, int this_hop,
		 piece my_king, moves *move_list, direction backward_left,
		 direction backward_right, direction forward_left,
		 direction forward_right, occupant orig_board[33]) {
  boolean  more_hops=false, jump_found=false;
  square jump_over, jump_to;
  jump_over = adjacent_squares[jump_from][the_direction];
  jump_to = adjacent_squares[jump_over][the_direction];
  if(board[jump_to] == off_board || board[jump_over] == off_board ||
     board[jump_over] == empty)
    return false;
  if((color_of(board[jump_over]) != color_of(board[jump_from])) &&
	 (board[jump_to] == empty)) {
      jump_found = true;
      hops_so_far->is_jump = true;
      hops_so_far->num_hops = this_hop + 1;
      hops_so_far->hops[this_hop].hop_from = jump_from;
      hops_so_far->hops[this_hop].hop_over = jump_over;
      hops_so_far->hops[this_hop].hop_to = jump_to;
      if((board[jump_from] != my_king) && (last_rank[jump_to])) {
	more_hops = false;
      } else {
	board[jump_to] = board[jump_from];
	board[jump_from] = empty;
	board[jump_over] = empty;
	generate_jumps_helper(board, jump_to, hops_so_far,
			      this_hop+1,forward_right,
			      forward_left, backward_right,
			      backward_left, my_king, move_list,
			      orig_board);
	if(hops_so_far->num_hops > this_hop+1) {
	    more_hops=true;
	}
      }
      if(!more_hops) {
	if(jump_found) {
	  move_list->list[move_list->length] = *hops_so_far;
	   undo_one_jump(board, hops_so_far->hops[this_hop], orig_board);
	  move_list->length++;
	} else {
	  move_list->length = 0;
	  move_list->list[move_list->length] = *hops_so_far;
	  jump_found = true;
	}
      }
      undo_one_jump(board, hops_so_far->hops[this_hop], orig_board);
  }
  return jump_found;
}

/*-----------------------------------------------------------------------*/
boolean generate_jumps_helper(occupant board[33], square the_square,
	move *jump_so_far,int this_hop, direction forward_right,
	direction forward_left, direction backward_right,
	direction backward_left, piece my_king, moves *move_list,
	occupant orig_board[33]) {
   boolean jump_found = false;
   try_jump(board, the_square, forward_left, jump_so_far, this_hop,
	my_king, move_list, backward_left, backward_right, forward_left,
	forward_right, orig_board);
   try_jump(board, the_square, forward_right, jump_so_far, this_hop,
	my_king, move_list, backward_left, backward_right, forward_left,
	forward_right, orig_board);
   if(board[the_square] == my_king) {
      try_jump(board, the_square, backward_left, jump_so_far, this_hop,
	     my_king, move_list, backward_left, backward_right, forward_left,
	     forward_right, orig_board);
       try_jump(board, the_square, backward_right, jump_so_far, this_hop,
	     my_king, move_list, backward_left, backward_right, forward_left,
	     forward_right, orig_board);
   }
   /* the following not in original code */
   return jump_found;
}

/*-------------------------------------------------------------------------*/
void undo_board(occupant board_to_fix[33], occupant original_board[33]) {
  int i;
  for(i=0; i<33; i++) {
      board_to_fix[i] = original_board[i];
  }
}


/*-------------------------------------------------------------------------*/
void undo_one_jump(occupant board[33], hop one_hop, occupant orig_board[33]) {
  if(board[one_hop.hop_to] != empty) {
    board[one_hop.hop_from] = board[one_hop.hop_to];
    board[one_hop.hop_over] = orig_board[one_hop.hop_over];
    board[one_hop.hop_to] = empty;
  }
}

/*--------------------------------------------------------------------------*/
void generate_jumps(game *the_game, moves *the_moves) {
  piece my_pieces[2];
  piece other_pieces[2];
  piece my_king;
  square my_last_rank[4];
  direction forward_left, forward_right, backward_left, backward_right;
  move empty_jump;
  int i, rank_num;
  occupant copy_board[33];
  if(the_game->to_move == black) {
    my_pieces[0] = black_man;
    my_pieces[1] = black_king;
    other_pieces[0] = white_man;
    other_pieces[1] = white_king;
    my_king = black_king;
    for(i=0, rank_num = 1; i<4;i++, rank_num++) {
      my_last_rank[i]=rank_num;
    }
    forward_left = down_right;
    forward_right = down_left;
    backward_left = up_right;
    backward_right = up_left;
  } else {
    my_pieces[0] = white_man; my_pieces[1] = white_king;
    other_pieces[0] = black_man; other_pieces[1] = black_king;
    my_king = white_king;
    for(i=0, rank_num = 29; i<4;i++, rank_num++) {
      my_last_rank[i]=rank_num;
    }
    forward_left = up_left;
    forward_right = up_right;
    backward_left = down_left;
    backward_right = down_right;
  }
  empty_jump.is_jump = true;
  the_moves->length = 0;
  for(i=0; i<33; i++) {
      copy_board[i] = the_game->board[i];
  }
  for(i=1; i<33; i++) {
    if(the_game->board[i]==my_pieces[0] || the_game->board[i]==my_pieces[1]) {
      generate_jumps_helper(the_game->board, i, &empty_jump, 0,
	     forward_right, forward_left,backward_right, backward_left,
	     my_king, the_moves, copy_board);
    }
  }
}

/*----------------------------------------------------------------------------*/
void apply_move(game *the_game, move the_move) {
  hop the_hop;
  square end_square;
  int i;
  if(the_move.is_jump) {
    for(i=0; i<the_move.num_hops; i++) {
	the_hop = the_move.hops[i];
	the_game->board[the_hop.hop_to] = the_game->board[the_hop.hop_from];
	the_game->board[the_hop.hop_over] = empty;
	the_game->board[the_hop.hop_from] = empty;
    }
    end_square = the_move.hops[the_move.num_hops - 1].hop_to;
  } else {
    the_game->board[the_move.move_to] = the_game->board[the_move.move_from];
    the_game->board[the_move.move_from] = empty;
    end_square = the_move.move_to;
  }
  if(the_game->board[end_square] == man_of_color(the_game->to_move) &&
	in_last_rank_of(end_square,opposite_color(the_game->to_move)) ) {
    the_game->board[end_square] = king_of_color(the_game->to_move);
  }
  the_game->to_move = opposite_color(the_game->to_move);
}

/*--------------------------------------------------------------------------*/
/* not for situations for which a jump is required.  Only uses moves (not   */
/* jumps in calculating mobility. INCLUDE JUMP DEEPENING IN MINIMAX CODE!   */
score static_evaluation(game *the_game, int *prior_mobility, int depth) {
    int position_weight_score = 0;
    square the_square;
    direction dir;
    color the_color;
    move try_move;
    score the_score;
    int current_mobility = 0;
    int black_men_count =0;
    int black_king_count = 0;
    int white_men_count = 0;
    int white_king_count =0;
    int bridge_score=0;
    int mobility_diff = 0;
    assert(valid_game(*the_game));
    /* only takes moves (not jumps) into account for mobility.*/
    try_move.is_jump=false;
    for(the_square=1; the_square<33; the_square++) {
      switch(the_game->board[the_square]) {
	case black_man:
	  black_men_count++;
	  position_weight_score += position_weight[black_man][the_square];
	  break;
	case white_man:
	  white_men_count++;
	  position_weight_score += position_weight[white_man][the_square];
	  break;
	case black_king:
	  black_king_count++;
	  position_weight_score += position_weight[black_king][the_square];
	  break;
	case white_king:
	  white_king_count++;
	  position_weight_score += position_weight[white_king][the_square];
	  break;
	case empty:
	  try_move.move_to = the_square;
	  for(dir=up_left; dir<=down_right; dir++) {
	      try_move.move_from = adjacent_squares[the_square][dir];
	      if((the_game->board[try_move.move_from] != empty) &&
		  ( the_game->board[try_move.move_from] != off_board)) {
		the_color = color_of(the_game->board[try_move.move_from]);
		if(the_color == the_game->to_move) {
		    if(valid_move_test(the_game->board,the_color,try_move)) {
			current_mobility++;
		    }
		}
	      }
	  }
	  break;
	default:
	  assert(false);
      }
    }
    if((white_men_count > 1) && (the_game->board[1] == black_man) &&
      (the_game->board[3]==black_man)) {
	bridge_score++;
    }
    if((black_men_count>1) && (the_game->board[30]==white_man) &&
		(the_game->board[32] == white_man)) {
	bridge_score--;
    }
    the_score.black_pieces = (black_men_count * MAN_VALUE) +
	(black_king_count * KING_VALUE);
    the_score.white_pieces = (white_men_count * MAN_VALUE) +
	(white_king_count * KING_VALUE);
    /* will this work--does the order of comparison matter */
    /* is this the way that checkers ends?????????         */
    if(/*the_score.black_pieces == 0 || */(current_mobility == 0 &&
       the_game->to_move == black)) {
      the_score.status = white_win;
      the_score.win_depth = depth;
    } else if(/*the_score.white_pieces == 0 || */(current_mobility == 0 &&
       the_game->to_move == white)) {
      the_score.status = black_win;
      the_score.win_depth = depth;
    } else {      /* neither is out of pieces or out of moves */
      the_score.status = unfinished;
      if(the_game->to_move == black) {
	mobility_diff =current_mobility - *prior_mobility;
      } else {
	mobility_diff = *prior_mobility -  current_mobility;
      }
      the_score.mobility = mobility_diff * MOBILITY_VALUE +
			   bridge_score * BRIDGE_VALUE +
			   position_weight_score * POSITION_VALUE;
    }
    *prior_mobility = current_mobility;
    return the_score;
}


/*-------------------------------------------------------------------------*/
score white_win_game(int depth) {
  score my_score;
  my_score.status=white_win;
  my_score.win_depth = depth;
  return my_score;
}

/*-------------------------------------------------------------------------*/
score black_win_game(int depth) {
  score my_score;
  my_score.status=black_win;
  my_score.win_depth = depth;
  return my_score;
}

/*--------------------------------------------------------------------------*/
/*        1-left is better for black, -1 right is better for black, 0 equal */
int compare_scores(score left, score right) {
  int total_piece_diff;
  int total_mobility;
  assert(valid_score(left) && valid_score(right));
  switch(left.status) {
    case black_win:
      switch(right.status) {
	case unfinished:
	case white_win:
	  return 1;
	case black_win:
	  return (sign(right.win_depth - left.win_depth));
      }
      break;
    case white_win:
      switch(right.status) {
	case black_win:
	  return -1;
	case unfinished:
	  return -1;
	case white_win:
	  return(sign(left.win_depth - right.win_depth));
      }
      break;
    case unfinished:
      switch(right.status) {
	case black_win:
	  return -1;
	case white_win:
	  return 1;
	case unfinished:      /* both unfinished */
	  total_piece_diff = (left.black_pieces - left.white_pieces) -
			 (right.black_pieces - right.white_pieces);
	  if(total_piece_diff > 0) {
	    return 1;
	  } else if(total_piece_diff <0) {
	    return -1;
	  } else {  /* both unfinished, same piece_diff */
		    /* now compare ratios               */
	    if((left.black_pieces != right.black_pieces) &&
		   (left.black_pieces != left.white_pieces)) {
		  /* look at ratios */
	      if(left.black_pieces > left.white_pieces) {
		/* more black pieces->favor score with less blacks */
		return(sign(right.black_pieces - left.black_pieces));
	      } else if (left.black_pieces < left.white_pieces) {
		/* more white pieces->favor score with more blacks */
		return(sign(left.black_pieces - right.black_pieces));
	      }
	    } else {
	      /* look at mobility */
	      total_mobility = left.mobility - right.mobility;
	      if(total_mobility > 0) {
		return 1;
	      } else if(total_mobility<0) {
		return -1;
	      } else {
		return 0;
	      }
	    }
	  }
      }
      break;
  }
  return 0;
}

/*--------------------------------------------------------------------------*/
score test_win(game position) {
  score static_score;
  moves move_list;
  generate_jumps(&position, &move_list);
  if(move_list.length == 0) {
    generate_non_jumps(&position, &move_list);
  }
  if(move_list.length == 0) {
    if(position.to_move == black) {
      static_score.status = white_win;
    } else {
      static_score.status = black_win;
    }
  } else {
    static_score.status = unfinished;
  }
  return static_score;
}

/*--------------------------------------------------------------------------*/
/*                           INPUT/OUTPUT FUNCTIONS                         */
/*--------------------------------------------------------------------------*/
void print_color(color the_color) {
    assert(valid_color(the_color));
    switch (the_color) {
	case black:
	    printf("black");
	    break;
	case white:
	    printf("white");
	    break;
    }
}

/*-------------------------------------------------------------------------*/
void print_board_number(int board_num) {
  if(board_num >= 10) {
    printf("%d", board_num);
  } else {
    printf("-%d", board_num);
  }
}

/*------------------------------------------------------------------------*/
void print_border_1(int count) {
  int c;
  for(c=3; c>=0; c--) {
    printf("+---+-");
    print_board_number(count);
    count++;
  }
  printf("+\n");
}

/*------------------------------------------------------------------------*/
void print_border_2(int count) {
  int c;
  for(c=3; c>=0; c--) {
    printf("+-");
    print_board_number(count);
    printf("+---");
    count++;
  }
  printf("+\n");
}

/*--------------------------------------------------------------------------*/
void print_game(game the_game) {
    int r, c;
    int count= 1;
    assert(valid_game(the_game));
    for (r =0; r < 4; r++) {
	print_border_1(count);
	for (c = 3; c >=0; c--) {
	  printf("|///| %c ", print_occupant(count, the_game));
	  count++;
	}
	printf("|\n");
	print_border_2(count);
	for(c=3; c>=0; c--) {
	  printf("| %c |///", print_occupant(count, the_game));
	  count++;
	}
	printf("|\n");
    }
    printf("+---+---+---+---+---+---+---+---+\n");
    print_color(the_game.to_move);
    printf("'s move\n");
}

/*---------------------------------------------------------------------------*/
char print_occupant(int position, game the_game) {
  switch (the_game.board[position]) {
    case empty:
      return(' ');
    case black_man:
      return('b');
    case white_man:
      return('w');
    case black_king:
      return('B');
    case white_king:
      return('W');
    default:
      return('*');
  }
}

/*--------------------------------------------------------------------------*/
void print_move(move the_move) {
  int i;
  hop the_hop;
  if(the_move.is_jump) {
    for(i=0; i<the_move.num_hops; i++) {
      the_hop = the_move.hops[i];
      printf("%d-", the_hop.hop_from);
    }
    printf("%d\n", the_hop.hop_to);
  } else {
    printf("%d-%d\n", the_move.move_from, the_move.move_to);
  }
}

/*----------------------------------------------------------------------------*/
/* THIS FUNCTION IS NEVER CALLED                                              */
void print_score(score the_score) {
    assert(valid_score(the_score));
    printf("{");
    switch (the_score.status) {
      case unfinished:
	printf("unfinished");
	break;
      case black_win:
	printf("black_win at depth %d", the_score.win_depth);
	break;
      case white_win:
	printf("white_win at depth %d", the_score. win_depth);
	break;
    }
    printf(", %d, %d, %d}\n", the_score.black_pieces, the_score.white_pieces,
	   the_score.mobility);
}

/*----------------------------------------------------------------------------*/
/* THIS FUNCTION IS NEVER CALLED. CAN BE MODIFIED FOR GAME I/O WHEN RESTARTING*/
/* A STOPPED GAME.                                                            */
boolean scan_game(game *the_game) {
  int return_code, i;
  char buffer[500];
  return_code = scanf("%s", buffer);
  if (return_code == EOF || strlen(buffer) != 32)
    return false;
  the_game->board[0] = off_board;
  for(i=0; i<32; i++) {
      switch(buffer[i]) {
	case 'b':
	  the_game->board[i+1] = black_man;
	  break;
	case 'B':
	  the_game->board[i+1] = black_king;
	  break;
	case 'w':
	  the_game->board[i+1] = white_man;
	  break;
	case 'W':
	  the_game->board[i+1] = white_king;
	  break;
	case '-':
	  the_game->board[i+1] = empty;
	  break;
	default:
	  return false;
      }
  }
  return (scan_color(&(the_game->to_move)));
}

/*----------------------------------------------------------------------------*/
boolean scan_color(color *the_color) {
    boolean result;
    int return_code;
    char buffer[100];
    return_code = scanf("%s", buffer);
    if (return_code == EOF)
	result = false;
    else {
	int i;
	for (i = 0; buffer[i] != '\0'; i++)
	    buffer[i] = (char) tolower((int) buffer[i]);
	if (!strcmp(buffer, "black") || !strcmp(buffer, "b")) {
	    *the_color = black;
	    result = true;
	} else if (!strcmp(buffer, "white") || !strcmp(buffer, "w")) {
	    *the_color = white;
	    result = true;
	} else
	    result = false;
    }
    return result;
}

/*---------------------------------------------------------------------*/
/* only works for positive numbers */
int string_to_integer(int start_index, int end_index, char buffer[]) {
  int i, total=0;
  for(i=start_index; i<= end_index; i++) {
      total *= 10;
      total += (buffer[i] - '0');
  }
  return total;
}

/*---------------------------------------------------------------------*/
/* returns -1 if char is not found.  Returns index of ch if found */
int find_char(char ch, int start_index, char buffer[]) {
  int i;
  for(i=start_index; buffer[i] != '\0'; i++) {
      if(buffer[i] == ch)
	return i;
  }
  return -1;
}

/*----------------------------------------------------------------------*/
/* return -1 if can't find hop_over-but shouldn't happen because        */
/* we already checked if it was on the move list                        */
square find_hop_over(int jump_from, int jump_to) {
  direction dir;
  for(dir=up_left; dir<=down_right; dir++) {
      if(adjacent_squares[adjacent_squares[jump_from][dir]][dir]
	 == jump_to) {
	return(adjacent_squares[jump_from][dir]);
      }
  }
  return -1;
}


/*----------------------------------------------------------------------*/
/* Need to fix for jumps !!!! PARSES USER'S INPUT                       */
boolean scan_move(move *the_move, moves move_list) {
    int return_code;
    char buffer[1000];
    move try_move;
    int start_index, dash_index, num, end_index;
    int num_array[30];                  /* max numbers is in 12 hops=25 */
    int effective_size, i, count;
    return_code = scanf("%s", buffer);
    if(return_code == EOF)
      return false;
    start_index = 0;
    effective_size = 0;
    while(true) {
	dash_index = find_char('-', start_index, buffer);
	end_index = dash_index -1;
	if(dash_index == -1)
	  end_index = strlen(buffer) - 1;
	num = string_to_integer(start_index, end_index, buffer);
	num_array[effective_size] = num;
	effective_size++;
	if(dash_index == -1)
	  break;
	start_index = dash_index+1;
    }
    if(effective_size == 0 || effective_size == 1) {
      return false;              /* no move fits into 0 or 1 num */
    }
    /* either simple move or 1 jump */
    if(effective_size == 2) {
      try_move.is_jump = false;
      try_move.move_from = num_array[0];
      try_move.move_to = num_array[1];
      if(move_in_generated_list(try_move, move_list)) {
	*the_move = try_move;
	return true;
      } else {
	try_move.is_jump = true;
	try_move.num_hops = 1;
	try_move.hops[0].hop_from = num_array[0];
	try_move.hops[0].hop_to = num_array[1];
	if(move_in_generated_list(try_move, move_list)) {
	  try_move.hops[0].hop_over = find_hop_over(num_array[0],
						    num_array[1]);
	  *the_move = try_move;
	  return true;
	} else {
	  return false;
	}
      }
    }
    /* else at least 2 jumps */
    try_move.is_jump = true;
    try_move.num_hops = effective_size -1;
    count=0;
    /* num_hops is effective_size -1 */
    for(i=0; i<effective_size-1; i++) {
	try_move.hops[i].hop_from = num_array[count];
	try_move.hops[i].hop_to = num_array[count+1];
	try_move.hops[i].hop_over = find_hop_over(num_array[count],
						  num_array[count+1]);
	count++;
    }
    if(move_in_generated_list(try_move, move_list)) {
      *the_move = try_move;
      return true;
    } else {
      return false;
    }
}

/******************************************************************************/
/*                                                                            */
/*                                Checkers Game                                */
/*                                ------------                                */
/*                                                                            */
/* Written by: John Thornley, Computer Science Dept., Caltech.                */
/* Last modified: Monday 24th January 1994.                                   */
/*                                                                            */
/* Purpose: Program to play a complete game of Checkers with the user.         */
/*                                                                            */
/******************************************************************************/

void print_moves(moves move_list) {
  int i;
  if(move_list.length > 1) {
      printf("possible moves are:\n");
  } else {
      printf("possible move is:\n");
  }
  for(i=0; i<move_list.length; i++) {
    printf("    ");
    print_move(move_list.list[i]);
  }
}

/*----------------------------------------------------------------------------*/
/*                                MAIN PROGRAM                                */
/*----------------------------------------------------------------------------*/
int max_depth;

void main() {
    boolean successful, must_jump;
    int     return_code, parity=0;
    color   players_color;
    game    position;
    score   static_score;
    move chosen_move;
    moves move_list;
    move_list.length = 0;

    setvbuf(stdout, NULL, _IOLBF,80);

    printf("Checkers Game\n");
    printf("------------\n");
    do {
	printf("Play black or white (black moves first)? ");
	successful = scan_color(&players_color);
    } while (!successful);
    do {
      if(players_color==black)
	parity=1;
      printf("Maximum search depth [1-10]? ");
      return_code = scanf("%i", &max_depth);
    } while (return_code == EOF || max_depth < 1 || 10 < max_depth );
    start_game(&position);
    printf("\n");
    print_game(position);
    printf("\n");
    static_score = test_win(position);
    while (static_score.status == unfinished) {
	must_jump = true;
	move_list.length = 0;
	if (position.to_move == players_color) {
		generate_jumps(&position, &move_list);
		if(move_list.length == 0) {
		  must_jump = false;
		  generate_non_jumps(&position, &move_list);
		  /* length should not be 0 or stopped earlier */
		}
		while(true) {
		  printf("Enter your move");
		  if(must_jump) printf(" (you must jump)");
		  printf(": ");
		  if(scan_move(&chosen_move, move_list))
		    break;
		  printf("Invalid move, ");
		  print_moves(move_list);
		}
	} else {
	    printf("Program choosing move:\n");
	    generate_jumps(&position, &move_list);
	    if(move_list.length == 0) {
	      generate_non_jumps(&position, &move_list);
	    }
	    if(move_list.length == 1){
	      chosen_move = move_list.list[0];
	    } else {
	       Minimax(&position, &chosen_move, parity);
	    }
	    printf("Program chooses ");
	    print_move(chosen_move);
	}
	apply_move(&position, chosen_move);
	static_score = test_win(position);
	printf("\n");
	print_game(position);
	printf("\n");
    }
    if ((static_score.status == black_win && players_color == black) ||
	     (static_score.status == white_win && players_color == white))
	printf("You win.\n");
    else
	printf("Program wins.\n");
    sleep (1);
}
