/* dame.c    main, drawing, menus, dialogs */

/*
 * Dame:     a point and click checkerboard
 * (C):      1998  Peter Chiocchetti  <girbal@super.tacheles.de>
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#define DELIMITER " "


#include <dame.h>
#include <dirent.h>
#include <gdk_imlib.h>


static GtkWidget *app;
static GtkWidget *board_prefs_dialog, *game_prefs_dialog;
static GtkWidget *omenu_bl, *omenu_wl;
static GtkWidget *draw_area, *vb;
static GdkImlibImage *image;
static GdkPixmap *stones;
static int stone_size;

static struct {
  char *scenario;
  int   make_it_default;
} selected_scenario = {0, 0};


/*
 * Drawing, events, ...
 */

void
draw_cell (int x, int y)
{
  gdk_draw_pixmap (draw_area->window, draw_area->style->black_gc, stones,
		   (board[x][y].colour) * stone_size, 0, x * stone_size,
		   y * stone_size, stone_size, stone_size);
}

static void
paint (GdkRectangle *area)
{
  int x1, y1, x2, y2, x, y;
	
  x1 = area->x / stone_size;
  y1 = area->y / stone_size;
  x2 = (area->x + area->width) / stone_size;
  y2 = (area->y + area->height) / stone_size;

  for (x = x1; x <= x2; x++) {
    for (y = y1; y <= y2; y++) {
      draw_cell (x, y);
    }
  }
}

static gint
expose_event (GtkWidget *widget, GdkEventExpose *event)
{
    paint (&event->area);
    return TRUE;
}

static void
button_event (GtkWidget *widget, GdkEventButton *event)
{
  int button;
  int x, y;

  button = event->button;
  x      = event->x / stone_size;
  y      = event->y / stone_size;

  if (board[x][y].colour > 0) {
    switch(button) {
    case 1:
    case 2:
    case 3:
      turn_trap (x, y);
      break;
    default:
      g_assert_not_reached();
    }
  }
}


/*
 * Board setup
 */

static void
empty_board (void)
{
  int x, y;

  for (y = 0; y < CELLS; y++) {
    for (x = 0; x < CELLS; x++)
      board[x][y].colour = (x+y) % 2;
  }
}

static void
fill_board (void)
{
  int x, y, n;
  n = 0 ;

  for (y = 0; y < CELLS; y++) {
    for (x = 0; x < CELLS; x++) {
      board[x][y].colour = ((x+y) % 2);
      board[x][y].number = 0;
      if ((x+y) % 2 == 1) {
	if (y <= (CELLS / 3))
	  board[x][y].colour += (direction == North ? 2 : 1);
	else if (y >= (CELLS - CELLS / 3 - 1))
	  board[x][y].colour += (direction == North ? 1 : 2);
	board[x][y].number = (direction == North
			      ? CELLS * CELLS / 2 - n : n + 1);
	n++;
      }
    }
  }
}

static void
load_scenario (void)
{
  char *tmp, *fn;
  GdkColor bgcolor;
  GdkImage *tmpimage;

  tmp = g_strconcat ("dame/", selected_scenario.scenario, NULL);
  fn = gnome_unconditional_pixmap_file (tmp);
  g_free (tmp);

  if (!g_file_exists (fn)) {
    fprintf (stderr,
	     (_("Dame : Could not find the \'%s\' theme.\n")), fn);
    exit (1);
  }

  if (image)
    gdk_imlib_destroy_image (image);
  image = gdk_imlib_load_image (fn);
  gdk_imlib_render (image, image->rgb_width, image->rgb_height);

  stones = gdk_imlib_move_image (image);
  tmpimage = gdk_image_get (stones, 0, 0, 1, 1);
  bgcolor.pixel = gdk_image_get_pixel (tmpimage, 0, 0);
  gdk_window_set_background (draw_area->window, &bgcolor);
  gdk_image_destroy (tmpimage);
  g_free (fn);

  stone_size = image->rgb_height;
  gtk_widget_set_usize (draw_area, CELLS  * stone_size,
			CELLS * stone_size); 
  gtk_widget_draw (draw_area, NULL);
}

static void
create_board (void)
{
  gtk_widget_push_visual (gdk_imlib_get_visual ());
  gtk_widget_push_colormap (gdk_imlib_get_colormap ());
  draw_area = gtk_drawing_area_new ();
  gtk_widget_pop_colormap ();
  gtk_widget_pop_visual ();
  gtk_widget_set_events (draw_area,
			 GDK_EXPOSURE_MASK | GDK_BUTTON_PRESS_MASK);

  gtk_box_pack_start_defaults (GTK_BOX (vb), draw_area);
  gtk_widget_realize (draw_area);
  gtk_widget_show (draw_area);

  gtk_signal_connect (GTK_OBJECT (draw_area), "expose_event",
		      GTK_SIGNAL_FUNC (expose_event), NULL);
  gtk_signal_connect (GTK_OBJECT (draw_area), "button_press_event",
		      GTK_SIGNAL_FUNC (button_event), NULL);

  load_scenario ();
}

static void
set_board (GtkWidget *widget, gpointer data)
{
  selected_scenario.scenario = strdup (data);
  gnome_dialog_set_default (GNOME_DIALOG (board_prefs_dialog), 0);
}

static void
set_board_def (GtkWidget *widget, gpointer data)
{
  selected_scenario.make_it_default = GTK_TOGGLE_BUTTON (widget)->active;
  gnome_dialog_set_default (GNOME_DIALOG (board_prefs_dialog), 0);
}

static int
bp_close_callback (GnomeDialog *dialog, gpointer data)
{
  board_prefs_dialog = 0;
  return FALSE;
}

static void
bp_clicked_callback (GnomeDialog *dialog, gint button, gpointer data)
{
  switch (button) {
  case 0:
    if (selected_scenario.scenario) {
      load_scenario ();
      if (selected_scenario.make_it_default) {
	gnome_config_set_string ("/dame/Preferences/Scenario",
				 selected_scenario.scenario);
	gnome_config_sync ();
      }
    }
    break;
  case 1:
    gnome_dialog_close (GNOME_DIALOG (board_prefs_dialog));
    break;
  default:
    g_assert_not_reached();
  }
}

static void
free_str (GtkWidget *widget, gpointer data)
{
  free (data);
}

static void
fill_menu (GtkWidget *menu)
{
  struct dirent *e;
  char *dname = gnome_unconditional_pixmap_file ("dame/");
  DIR *dir;
  int itemno = 0;

  dir = opendir (dname);
  if (!dir)
    return;

  while ((e = readdir (dir)) != NULL){
    GtkWidget *item;
    char *s = strdup (e->d_name);
    if (!(strstr (e->d_name, ".png")  ||
	  strstr (e->d_name, ".gif")  ||
	  strstr (e->d_name, ".tiff") ||
	  strstr (e->d_name, ".jpeg") ||
	  strstr (e->d_name, ".xpm"))) {
      free (s);
      continue;
    }

    item = gtk_menu_item_new_with_label (s);
    gtk_widget_show (item);
    gtk_menu_append (GTK_MENU (menu), item);
    gtk_signal_connect (GTK_OBJECT (item), "activate",
			GTK_SIGNAL_FUNC (set_board), s);
    gtk_signal_connect (GTK_OBJECT (item), "destroy",
			GTK_SIGNAL_FUNC (free_str), s);
    if (!strcmp (selected_scenario.scenario, s))
      gtk_menu_set_active (GTK_MENU (menu), itemno);
    itemno++;
  }
}

static void
board_prefs_callback (GtkWidget *widget, gpointer data)
{
  GtkWidget *menu, *omenu, *l, *hb, *cb;

  if (board_prefs_dialog)
    return;

  selected_scenario.make_it_default = 0;
  board_prefs_dialog = gnome_dialog_new (_("Board"),
					 GNOME_STOCK_BUTTON_APPLY,
					 GNOME_STOCK_BUTTON_CLOSE,
					 NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (board_prefs_dialog),
			   GTK_WINDOW (app));
  gtk_signal_connect (GTK_OBJECT (board_prefs_dialog), "close",
		      GTK_SIGNAL_FUNC (bp_close_callback), NULL);
  gtk_signal_connect (GTK_OBJECT (board_prefs_dialog), "clicked",
		      GTK_SIGNAL_FUNC (bp_clicked_callback), NULL);

  omenu = gtk_option_menu_new ();
  menu = gtk_menu_new ();
  fill_menu (menu);
  gtk_widget_show (omenu);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);

  hb = gtk_hbox_new (FALSE, FALSE);
  gtk_widget_show (hb);

  l = gtk_label_new (_("Select scenario: "));
  gtk_widget_show (l);

  gtk_box_pack_start_defaults (GTK_BOX (hb), l);
  gtk_box_pack_start_defaults (GTK_BOX (hb), omenu);

  cb = gtk_check_button_new_with_label (_("Make it the default scenario"));
  gtk_signal_connect (GTK_OBJECT (cb), "clicked",
		      GTK_SIGNAL_FUNC (set_board_def), NULL);
  gtk_widget_show (cb);

  vb = GNOME_DIALOG (board_prefs_dialog)->vbox;
  gtk_widget_show (vb);

  gtk_box_pack_start_defaults (GTK_BOX (vb), hb);
  gtk_box_pack_start_defaults (GTK_BOX (vb), cb);

  gtk_widget_show (board_prefs_dialog);
}


/*
 * game setup
 */

static void
new_game (void)
{
  if (!strcmp (players.white, username)) {
    direction = South;
    if (!strcmp (players.black, username))
      direction = North;
  }
  else
    direction = North;

  touched[0]      = 0; /* This means no.  */
  go_on           = 0;
  must_jump       = 0;
  game_over       = 0;
  players.current = Black;

  fill_board ();
  gtk_widget_draw (draw_area, NULL);
  if (verbose) {
    fprintf (stdout, ("\n\n"));
    fprintf (stdout, "%8s : %-8s\n", players.black, players.white);
  }
  gnome_appbar_set_status (GNOME_APPBAR (ab), _("Black's move"));
  start_talk ();
}

static int
gp_close_callback (GnomeDialog *dialog, gpointer data)
{
  if (players.black && players.white) {
    if (players.make_em_default) {
      gnome_config_set_string ("/dame/Preferences/Black Player",
			       (strcmp (players.black, username)
			       ? players.black : "human"));
      gnome_config_set_string ("/dame/Preferences/Black Level",
			       players.black_level);
      gnome_config_set_string ("/dame/Preferences/White Player",
			       (strcmp (players.white, username)
				? players.white : "human"));
      gnome_config_set_string ("/dame/Preferences/White Level",
			       players.white_level);
      gnome_config_sync ();
    }
    new_game ();
    game_prefs_dialog = 0;
    return FALSE;
  }
  else
    return TRUE;
}

static void
gp_clicked_callback (GnomeDialog *dialog, gint button, gpointer data)
{
  gnome_dialog_close (GNOME_DIALOG (game_prefs_dialog));
}

static void
new_game_callback (GtkWidget *widget, gpointer data)
{
  if (game_prefs_dialog)
    gnome_dialog_close (GNOME_DIALOG (game_prefs_dialog));
  else {
    stop_talk ();
    new_game ();
  }
}

static void
set_black (GtkWidget *widget, gpointer data)
{
  players.black = strdup (data);
  if (strcmp (players.black, username))
    gtk_widget_set_sensitive (GTK_WIDGET (omenu_bl), TRUE);
  else
    gtk_widget_set_sensitive (GTK_WIDGET (omenu_bl), FALSE);
}

static void
set_black_level (GtkWidget *widget, gpointer data)
{
  players.black_level = strdup (data);
}

static void
set_white (GtkWidget *widget, gpointer data)
{
  players.white = strdup (data);
  if (strcmp (players.white, username))
    gtk_widget_set_sensitive (GTK_WIDGET (omenu_wl), TRUE);
  else
    gtk_widget_set_sensitive (GTK_WIDGET (omenu_wl), FALSE);
}

static void
set_white_level (GtkWidget *widget, gpointer data)
{
  players.white_level = strdup (data);
}

static void
set_players_def (GtkWidget *widget, gpointer data)
{
  players.make_em_default = GTK_TOGGLE_BUTTON (widget)->active;
}

static void
fill_menu_b (GtkWidget *menu)
{
  int itemno = 0;
  char *token;
  char *running = gnome_config_get_string_with_default
    ("/dame/Preferences/Player List=human checkers simplech", NULL);

  while ((token = strsep (&running, DELIMITER)) != 0) {
    GtkWidget *item;
    char *s = strdup (strcmp (token, "human") ? token : username);
    item = gtk_menu_item_new_with_label (s);
    gtk_widget_show (item);
    gtk_menu_append (GTK_MENU(menu), item);
    gtk_signal_connect (GTK_OBJECT (item), "activate",
			GTK_SIGNAL_FUNC (set_black), s);
    gtk_signal_connect (GTK_OBJECT (item), "destroy",
			GTK_SIGNAL_FUNC (free_str), s);
    if (!strcmp (players.black, s))
      gtk_menu_set_active (GTK_MENU (menu), itemno);
    itemno++;
  }
}

static void
fill_menu_bl (GtkWidget *menu)
{
  int itemno;

  for (itemno = 0; itemno < 9; itemno++) {
    GtkWidget *item;
    char *s = (char *) malloc (3 * sizeof (char));
    sprintf (s, "%d", itemno + 1);
    item = gtk_menu_item_new_with_label (s);
    gtk_widget_show (item);
    gtk_menu_append (GTK_MENU (menu), item);
    gtk_signal_connect (GTK_OBJECT (item), "activate",
			GTK_SIGNAL_FUNC (set_black_level), s);
    gtk_signal_connect (GTK_OBJECT (item), "destroy",
			GTK_SIGNAL_FUNC (free_str), s);
    if (!strcmp (players.black_level, s))
      gtk_menu_set_active (GTK_MENU (menu), itemno);
  }
}

static void
fill_menu_w (GtkWidget *menu)
{
  int itemno = 0;
  char *token;
  char *running = gnome_config_get_string_with_default
    ("/dame/Preferences/Player List=human checkers simplech", NULL);

  while ((token = strsep (&running, DELIMITER)) != 0) {
    GtkWidget *item;
    char *s = strdup (strcmp (token, "human") ? token : username);
    item = gtk_menu_item_new_with_label (s);
    gtk_widget_show (item);
    gtk_menu_append (GTK_MENU (menu), item);
    gtk_signal_connect (GTK_OBJECT (item), "activate",
			GTK_SIGNAL_FUNC (set_white), s);
    gtk_signal_connect (GTK_OBJECT (item), "destroy",
			GTK_SIGNAL_FUNC (free_str), s);
    if (!strcmp (players.white, s))
      gtk_menu_set_active (GTK_MENU (menu), itemno);
    itemno++;
  }
}

static void
fill_menu_wl (GtkWidget *menu)
{
  int itemno;

  for (itemno = 0; itemno < 9; itemno++) {
    GtkWidget *item;
    char *s = (char *) malloc (3 * sizeof (char));
    sprintf (s, "%d", itemno + 1);
    item = gtk_menu_item_new_with_label (s);
    gtk_widget_show (item);
    gtk_menu_append (GTK_MENU (menu), item);
    gtk_signal_connect (GTK_OBJECT (item), "activate",
			GTK_SIGNAL_FUNC (set_white_level), s);
    gtk_signal_connect (GTK_OBJECT (item), "destroy",
			GTK_SIGNAL_FUNC (free_str), s);
    if (!strcmp (players.white_level, s))
      gtk_menu_set_active (GTK_MENU (menu), itemno);
  }
}

static void
game_prefs_callback (GtkWidget *widget, gpointer data)
{
  GtkWidget *menu_b, *omenu_b, *menu_bl, *l_b, *hb_b;
  GtkWidget *menu_w, *omenu_w, *menu_wl, *l_w, *hb_w;
  GtkWidget *cb;

  if (game_prefs_dialog)
    return;

  stop_talk ();
  players.make_em_default = 0;
  touched[0] = 0;
  empty_board ();
  gtk_widget_draw (draw_area, NULL);

  game_prefs_dialog = gnome_dialog_new (_("New Game"),
					GNOME_STOCK_BUTTON_OK,
					NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (game_prefs_dialog),
			   GTK_WINDOW (app));
  gtk_signal_connect (GTK_OBJECT (game_prefs_dialog), "close",
		      GTK_SIGNAL_FUNC (gp_close_callback), NULL);
  gtk_signal_connect (GTK_OBJECT (game_prefs_dialog), "clicked",
		      GTK_SIGNAL_FUNC (gp_clicked_callback), NULL);

  /* black Player */
  omenu_b = gtk_option_menu_new ();
  menu_b = gtk_menu_new ();
  fill_menu_b (menu_b);
  gtk_widget_show (omenu_b);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu_b), menu_b);

  omenu_bl = gtk_option_menu_new ();
  menu_bl = gtk_menu_new ();
  fill_menu_bl (menu_bl);
  gtk_widget_show (omenu_bl);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu_bl), menu_bl);

  if (strcmp (players.black, username))
    gtk_widget_set_sensitive (GTK_WIDGET (omenu_bl), TRUE);
  else
    gtk_widget_set_sensitive (GTK_WIDGET (omenu_bl), FALSE);

  hb_b = gtk_hbox_new (FALSE, FALSE);
  gtk_widget_show (hb_b);
  l_b = gtk_label_new (_("Select Black Player and Level: "));
  gtk_widget_show (l_b);
  gtk_box_pack_start_defaults (GTK_BOX (hb_b), omenu_b);
  gtk_box_pack_start_defaults (GTK_BOX (hb_b), omenu_bl);

  /* white Player */
  omenu_w = gtk_option_menu_new ();
  menu_w = gtk_menu_new ();
  fill_menu_w(menu_w);
  gtk_widget_show (omenu_w);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu_w), menu_w);

  omenu_wl = gtk_option_menu_new ();
  menu_wl = gtk_menu_new ();
  fill_menu_wl(menu_wl);
  gtk_widget_show (omenu_wl);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu_wl), menu_wl);

  if (strcmp (players.white, username))
    gtk_widget_set_sensitive (GTK_WIDGET (omenu_wl), TRUE);
  else
    gtk_widget_set_sensitive (GTK_WIDGET (omenu_wl), FALSE);

  hb_w = gtk_hbox_new (FALSE, FALSE);
  gtk_widget_show (hb_w);
  l_w = gtk_label_new (_("Select White Player and Level: "));
  gtk_widget_show (l_w);
  gtk_box_pack_start_defaults (GTK_BOX (hb_w), omenu_w);
  gtk_box_pack_start_defaults (GTK_BOX (hb_w), omenu_wl);

  /* both Players */
  cb = gtk_check_button_new_with_label (_("Make them the default Players"));
  gtk_signal_connect (GTK_OBJECT (cb), "clicked",
		      GTK_SIGNAL_FUNC (set_players_def), NULL);
  gtk_widget_show (cb);

  vb = GNOME_DIALOG(game_prefs_dialog)->vbox;
  gtk_widget_show (vb);

  gtk_box_pack_start_defaults (GTK_BOX (vb), l_b);
  gtk_box_pack_start_defaults (GTK_BOX (vb), hb_b);
  gtk_box_pack_start_defaults (GTK_BOX (vb), l_w);
  gtk_box_pack_start_defaults (GTK_BOX (vb), hb_w);
  gtk_box_pack_start_defaults (GTK_BOX (vb), cb);

  gtk_widget_show (game_prefs_dialog);
}


/*
 * What's it about ?
 */

static int
game_about_callback (GtkWidget *widget, gpointer data)
{
  GtkWidget *about;
  const gchar *authors[] = {
    "Peter Chiocchetti",
/*     "dame: Peter Chiocchetti", */
/*     "mc_checkers: John Thornley, David Sinclair", */
/*     "checkers: Lena Petrovic, John Langford", */
/*     "simplech: Martin Fierz", */
    NULL
  };

  about = gnome_about_new (_("Dame"), VERSION,
			   " 1998-1999",
 			   authors,
			   (_("...seduce into error "
			      "or hurry into miscalculation.\n"
			      "E. A. Poe on draughts")),
/*  			   "dame.png"); */
			   NULL);
  gnome_dialog_set_parent(GNOME_DIALOG(about), GTK_WINDOW(app));
  gtk_widget_show (about);
  return TRUE;
}


/*
 * Just Quit
 */

static int
game_quit_callback (GtkWidget *widget, gpointer data)
{
  if (verbose)
    fprintf (stdout, "\n");
  gtk_widget_destroy (app);
  gtk_main_quit ();

  return TRUE;
}


/*
 * Toolbar Declarations and Definitions
 */

GnomeUIInfo toolbar[] = {
  GNOMEUIINFO_ITEM_NONE (N_("New"),
			 N_("Start a new game"), new_game_callback),
  GNOMEUIINFO_ITEM_NONE (N_("New..."),
			 N_("New game with options"), game_prefs_callback),
  GNOMEUIINFO_ITEM_NONE (N_("Board..."),
			 N_("Change scenario"), board_prefs_callback),
  GNOMEUIINFO_ITEM_NONE (N_("Quit"),
			 N_("Just exit"), game_quit_callback),
  GNOMEUIINFO_ITEM_NONE (N_("About..."),
			 N_("Dame info"), game_about_callback),
  GNOMEUIINFO_END
};
 

/*
 * Command Line Parsing
 */

static gchar *fname;
static gchar *bname;
static gchar *blevel;
static gchar *wname;
static gchar *wlevel;

static const struct poptOption options[] = {
  { "black_player", 'b' , POPT_ARG_STRING, &bname  , 0,
    N_("Black Player"), N_("PROGRAM") },
  { "black_level" , 'B' , POPT_ARG_STRING, &blevel , 0,
    N_("Black Level") , N_("NUMBER")  },
  { "white_player", 'w' , POPT_ARG_STRING, &wname  , 0,
    N_("White Player"), N_("PROGRAM") },
  { "white_level" , 'W' , POPT_ARG_STRING, &wlevel , 0,
    N_("White Level") , N_("NUMBER")  },
  { "set_scenario", 's' , POPT_ARG_STRING, &fname  , 0,
    N_("Set Scenario"), N_("FILE")    },
  { "verbose"     , 'v' , POPT_ARG_NONE  , &verbose, 0,
    N_("Print Moves") , NULL          },
  { NULL          , '\0', 0              , NULL    , 0,
    NULL              , NULL          }
};


/*
 * Main function
 */

int
main (int argc, char *argv [])
{
  GtkStyle *style;
  GdkBitmap *mask;
  GdkPixmap *icon;

  bindtextdomain (PACKAGE, GNOMELOCALEDIR);
  textdomain (PACKAGE);

  gnome_init_with_popt_table ("dame", VERSION, argc, argv, options, 0, NULL);
  gdk_imlib_init ();

  /* Merge with defaults.  */
  username = g_get_user_name ();
  if (!bname)
    bname = gnome_config_get_string_with_default
      ("/dame/Preferences/Black Player=human", NULL);
  players.black = g_strdup (strcmp (bname, "human") ? bname : username);
  if (!blevel)
    blevel = gnome_config_get_string_with_default
      ("/dame/Preferences/Black Level=3", NULL);
  players.black_level = g_strdup (blevel);
  if (!wname)
    wname = gnome_config_get_string_with_default
      ("/dame/Preferences/White Player=checkers", NULL);
  players.white = g_strdup (strcmp (wname, "human") ? wname : username);
  if (!wlevel)
    wlevel = gnome_config_get_string_with_default
      ("/dame/Preferences/White Level=3", NULL);
  players.white_level = g_strdup (wlevel);
  if (!fname)
    fname = gnome_config_get_string_with_default
      ("/dame/Preferences/Scenario=official.png", NULL);
  selected_scenario.scenario = g_strdup (fname);

  /* Make program window. Fill it.  */
  app = gnome_app_new("dame", _("Dame"));
  gtk_window_set_policy (GTK_WINDOW (app), FALSE, FALSE, TRUE);
  gtk_signal_connect (GTK_OBJECT (app), "delete_event",
		      GTK_SIGNAL_FUNC (game_quit_callback), NULL);

  gnome_app_create_toolbar (GNOME_APP (app), toolbar);

  ab = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
  gnome_app_set_statusbar(GNOME_APP (app), GTK_WIDGET(ab));
  gnome_appbar_set_default (GNOME_APPBAR (ab), _("Welcome to Dame!"));
/*    gnome_app_install_menu_hints(GNOME_APP (app), toolbar); */

  vb = gtk_vbox_new (FALSE, 0);
  gnome_app_set_contents (GNOME_APP (app), vb);

  create_board ();

/*    gtk_widget_show (ab); */
  gtk_widget_show (vb);
  gtk_widget_show (app);

  empty_board ();

/*    g_free (fname); */
/*    g_free (bname); */
/*    g_free (blevel); */
/*    g_free (wname); */
/*    g_free (wlevel); */

  style = gtk_widget_get_style (app);
  icon = gdk_pixmap_create_from_xpm (app->window, &mask,
				     &style->bg[GTK_STATE_NORMAL],
				     "./dame.xpm");
  gdk_window_set_icon (app->window,
		       NULL,
		       icon,
		       mask);

  gtk_main ();
  return 0;
}
